import base64
import httplib

from django.contrib.auth.models import User
from django.test import TestCase

import mock

test_username = 'username'
test_password = 'password'

def mocked_authenticate(username, password):
    if username == test_username and password == test_password:
        return User(username='active')

def mocked_authenticate_inactive(username, password):
    if username == test_username and password == test_password:
        return User(username='inactive', is_active=False)

def basic_auth(username, password):
    return {'HTTP_AUTHORIZATION': 'Basic ' + base64.b64encode(':'.join([username, password]))}

@mock.patch('django.conf.settings.MIDDLEWARE_CLASSES',
            ('django.contrib.sessions.middleware.SessionMiddleware',
             'django.contrib.auth.middleware.AuthenticationMiddleware',
             'django_conneg.support.middleware.BasicAuthMiddleware',))
class BasicAuthTestCase(TestCase):
    def testOptionalWithout(self):
        response = self.client.get('/optional-auth/')
        self.assertEqual(response.status_code, httplib.OK)
        self.assertFalse(response.is_authenticated)

    @mock.patch('django_conneg.support.middleware.authenticate', mocked_authenticate)
    def testOptionalWithCorrect(self):
        response = self.client.get('/optional-auth/',
                                   **basic_auth(test_username, test_password))
        self.assertEqual(response.status_code, httplib.OK)
        self.assertTrue(response.is_authenticated)

    @mock.patch('django_conneg.support.middleware.authenticate', mocked_authenticate)
    def testOptionalWithIncorrect(self):
        response = self.client.get('/optional-auth/',
                                   **basic_auth(test_username, 'not-the-password'))
        self.assertEqual(response.status_code, httplib.UNAUTHORIZED)
        self.assertEqual(response['WWW-Authenticate'], 'Basic realm="restricted"')

    def testRequiredWithoutHTML(self):
        response = self.client.get('/login-required/', HTTP_ACCEPT='text/html')
        self.assertEqual(response.status_code, httplib.FOUND)
        self.assertEqual(response['Location'],
                         'http://testserver/login/?next=/login-required/')

    def testRequiredWithoutJSON(self):
        response = self.client.get('/login-required/', HTTP_ACCEPT='application/json')
        self.assertEqual(response.status_code, httplib.UNAUTHORIZED)
        self.assertEqual(response['WWW-Authenticate'], 'Basic realm="restricted"')

    @mock.patch('django_conneg.support.middleware.authenticate', mocked_authenticate)
    def testRequiredWithCorrect(self):
        response = self.client.get('/login-required/',
                                   **basic_auth(test_username, test_password))
        self.assertEqual(response.status_code, httplib.OK)
        self.assertTrue(response.is_authenticated)

    @mock.patch('django_conneg.support.middleware.authenticate', mocked_authenticate_inactive)
    def testRequiredWithCorrectInactive(self):
        response = self.client.get('/login-required/',
                                   **basic_auth(test_username, test_password))
        self.assertEqual(response.status_code, httplib.FORBIDDEN)



import itertools
import unittest

from django.http import HttpResponse

from django_conneg import http, views, decorators

class PriorityTestCase(unittest.TestCase):
    mimetypes = ('text/plain', 'application/xml', 'text/html', 'application/json')

    def getRenderer(self, format, mimetypes, name, priority):
        if not isinstance(mimetypes, tuple):
            mimetypes = (mimetypes,)
        def renderer(request, context, template_name):
            return HttpResponse('', mimetype=mimetypes[0])
        renderer.__name__ = 'render_%s' % mimetypes[0].replace('/', '_')
        renderer = decorators.renderer(format=format,
                                       mimetypes=mimetypes,
                                       priority=priority)(renderer)
        return renderer

    def getTestView(self, priorities):
        members = {}
        for i, (mimetype, priority) in enumerate(priorities.items()):
            members['render_%d' % i] = self.getRenderer(str(i), mimetype, str(i), priority)
        TestView = type('TestView',
                        (views.ContentNegotiatedView,),
                        members)
        return TestView

    def testEqualQuality(self):
        accept_header = ', '.join(self.mimetypes)
        accept = http.MediaType.parse_accept_header(accept_header)

        for mimetypes in itertools.permutations(self.mimetypes):
            renderers = tuple(self.getRenderer(str(i), mimetype, str(i), -i) for i, mimetype in enumerate(mimetypes))

            renderers = http.MediaType.resolve(accept, renderers)

            for renderer, mimetype in zip(renderers, mimetypes):
                self.assertEqual(iter(renderer.mimetypes).next(), http.MediaType(mimetype))

    def testEqualQualityView(self):
        accept_header = ', '.join(self.mimetypes)
        accept = http.MediaType.parse_accept_header(accept_header)

        for mimetypes in itertools.permutations(self.mimetypes):
            priorities = dict((mimetype, -i) for i, mimetype in enumerate(mimetypes))
            test_view = self.getTestView(priorities).as_view()
            renderers = http.MediaType.resolve(accept, test_view.conneg.renderers)

            for renderer, mimetype in zip(renderers, mimetypes):
                self.assertEqual(iter(renderer.mimetypes).next(), http.MediaType(mimetype))

    def testPrioritySorting(self):
        for mimetypes in itertools.permutations(self.mimetypes):
            priorities = dict((mimetype, -i) for i, mimetype in enumerate(mimetypes))
            test_view = self.getTestView(priorities).as_view()

            renderer_priorities = [renderer.priority for renderer in test_view.conneg.renderers]
            self.assertEqual(renderer_priorities, sorted(renderer_priorities, reverse=True))



if __name__ == '__main__':
    unittest.main()



import datetime
import httplib
import inspect
import itertools
import logging
import sys
import time
import urllib
import urlparse
import warnings

from django.core import exceptions
from django.views.generic import View
from django.utils.decorators import classonlymethod
from django import http
from django.template import RequestContext, TemplateDoesNotExist
from django.shortcuts import render_to_response
from django.utils.cache import patch_vary_headers

from django_conneg.conneg import Conneg
from django_conneg.decorators import renderer
from django_conneg.http import MediaType, HttpError, HttpNotAcceptable
from django_conneg.utils import utc

logger = logging.getLogger(__name__)

class BaseContentNegotiatedView(View):
    conneg = None
    context = None
    _default_format = None
    _force_fallback_format = None
    _format_override_parameter = 'format'
    
    template_name = None

    @classonlymethod
    def as_view(cls, **initkwargs):
        view = super(BaseContentNegotiatedView, cls).as_view(**initkwargs)
        view.conneg = Conneg(obj=cls)
        return view

    def dispatch(self, request, *args, **kwargs):
        # This is handy for the view to work out what renderers will
        # be attempted, and to manipulate the list if necessary.
        # Also handy for middleware to check whether the view was a
        # BaseContentNegotiatedView, and which renderers were preferred.
        if self.context is None:
            self.context = {'additional_headers': {}}
        self.request = request
        self.args = args
        self.kwargs = kwargs
        self.conneg = Conneg(obj=self)
        self.set_renderers(request)
        return super(BaseContentNegotiatedView, self).dispatch(request, *args, **kwargs)

    def set_renderers(self, request=None, context=None, template_name=None):
        """
        Makes sure that the renderers attribute on the request is up
        to date. renderers_for_view keeps track of the view that
        is attempting to render the request, so that if the request
        has been delegated to another view we know to recalculate
        the applicable renderers. When called multiple times on the
        same view this will be very low-cost for subsequent calls.
        """
        request, context, template_name = self.get_render_params(request, context, template_name)

        args = (self.conneg, context, template_name,
                self._default_format, self._force_fallback_format, self._format_override_parameter)
        if getattr(request, 'renderers_for_args', None) != args:
            fallback_formats = self._force_fallback_format or ()
            if not isinstance(fallback_formats, (list, tuple)):
                fallback_formats = (fallback_formats,)
            if request.REQUEST.get(self._format_override_parameter):
                format_override = request.REQUEST[self._format_override_parameter].split(',')
            else:
                format_override = None
            request.renderers = self.conneg.get_renderers(request=request,
                                                          context=context,
                                                          template_name=template_name,
                                                          accept_header=request.META.get('HTTP_ACCEPT'),
                                                          formats=format_override,
                                                          default_format=self._default_format,
                                                          fallback_formats=fallback_formats)
            request.renderers_for_view = args
        self.context['renderers'] = [self.renderer_for_context(request, r) for r in self.conneg.renderers]
        return request.renderers

    def get_render_params(self, request, context, template_name):
        if not template_name:
            template_name = self.template_name
            if isinstance(template_name, basestring) and template_name.endswith('.html'):
                template_name = template_name[:-5]
        return request or self.request, context or self.context, template_name

    def render(self, request=None, context=None, template_name=None):
        """
        Returns a HttpResponse of the right media type as specified by the
        request.
        
        context can contain status_code and additional_headers members, to set
        the HTTP status code and headers of the request, respectively.
        template_name should lack a file-type suffix (e.g. '.html', as
        renderers will append this as necessary.
        """
        request, context, template_name = self.get_render_params(request, context, template_name)

        self.set_renderers()

        status_code = context.pop('status_code', httplib.OK)
        additional_headers = context.pop('additional_headers', {})

        for renderer in request.renderers:
            response = renderer(request, context, template_name)
            if response is NotImplemented:
                continue
            response.status_code = status_code
            response.renderer = renderer
            break
        else:
            tried_mimetypes = list(itertools.chain(*[r.mimetypes for r in request.renderers]))
            response = self.http_not_acceptable(request, tried_mimetypes)
            response.renderer = None
        for key, value in additional_headers.iteritems():
            response[key] = value

        # We're doing content-negotiation, so tell the user-agent that the
        # response will vary depending on the accept header.
        patch_vary_headers(response, ('Accept',))
        return response

    def http_not_acceptable(self, request, tried_mimetypes, *args, **kwargs):
        response = http.HttpResponse("""\
Your Accept header didn't contain any supported media ranges.

Supported ranges are:

 * %s\n""" % '\n * '.join(sorted('%s (%s; %s)' % (f.name, ", ".join(m.value for m in f.mimetypes), f.format) for f in request.renderers if not any(m in tried_mimetypes for m in f.mimetypes))), mimetype="text/plain")
        response.status_code = httplib.NOT_ACCEPTABLE
        return response

    def head(self, request, *args, **kwargs):
        handle_get = getattr(self, 'get', None)
        if handle_get:
            response = handle_get(request, *args, **kwargs)
            response.content = ''
            return response
        else:
            return self.http_method_not_allowed(request, *args, **kwargs)

    def options(self, request, *args, **kwargs):
        response = http.HttpResponse()
        response['Accept'] = ','.join(m.upper() for m in sorted(self.http_method_names) if hasattr(self, m))
        return response

    @classmethod
    def parse_accept_header(cls, accept):
        warnings.warn("The parse_accept_header method has moved to django_conneg.http.MediaType")
        return MediaType.parse_accept_header(accept)

    def render_to_format(self, request=None, context=None, template_name=None, format=None):
        request, context, template_name = self.get_render_params(request, context, template_name)
        self.set_renderers()

        status_code = context.pop('status_code', httplib.OK)
        additional_headers = context.pop('additional_headers', {})

        for renderer in self.conneg.renderers_by_format.get(format, ()):
            response = renderer(request, context, template_name)
            if response is not NotImplemented:
                break
        else:
            response = self.http_not_acceptable(request, ())
            renderer = None

        response.status_code = status_code
        response.renderer = renderer
        for key, value in additional_headers.iteritems():
            response[key] = value
        return response

    def join_template_name(self, template_name, extension):
        """
        Appends an extension to a template_name or list of template_names.
        """
        if template_name is None:
            return None
        if isinstance(template_name, (list, tuple)):
            return tuple('.'.join([n, extension]) for n in template_name)
        if isinstance(template_name, basestring):
            return '.'.join([template_name, extension])
        raise AssertionError('template_name not of correct type: %r' % type(template_name))

    def renderer_for_context(self, request, renderer):
        return {'name': renderer.name,
                'priority': renderer.priority,
                'mimetypes': [m.value for m in renderer.mimetypes],
                'format': renderer.format,
                'url': self.url_for_format(request, renderer.format)}

    def url_for_format(self, request, format):
        qs = urlparse.parse_qs(request.META.get('QUERY_STRING', ''))
        qs['format'] = [format]
        return '?{0}'.format(urllib.urlencode(qs, True))



class ContentNegotiatedView(BaseContentNegotiatedView):
    @property
    def error_view(self):
        if not hasattr(self, '_error_view'):
            self._error_view = ErrorView.as_view()
        return self._error_view

    error_template_names = {httplib.NOT_FOUND: ('conneg/not_found', '404'),
                            httplib.FORBIDDEN: ('conneg/forbidden', '403'),
                            httplib.NOT_ACCEPTABLE: ('conneg/not_acceptable',),
                            httplib.BAD_REQUEST: ('conneg/bad_request', '400')}

    def dispatch(self, request, *args, **kwargs):
        try:
            return super(ContentNegotiatedView, self).dispatch(request, *args, **kwargs)
        except http.Http404, e:
            return self.error(request, e, args, kwargs, httplib.NOT_FOUND)
        except exceptions.PermissionDenied, e:
            return self.error(request, e, args, kwargs, httplib.FORBIDDEN)
        except HttpError, e:
            return self.error(request, e, args, kwargs, e.status_code)

    def http_not_acceptable(self, request, tried_mimetypes, *args, **kwargs):
        raise HttpNotAcceptable(tried_mimetypes)

    def error(self, request, exception, args, kwargs, status_code):
        method_name = 'error_%d' % status_code
        method = getattr(self, method_name, None)

        # See if we've got a dedicated handler for this status code
        if callable(method):
            return method(request, exception, *args, **kwargs)
        # Otherwise, if it's an HttpError, try to render it to an
        # appropriate template
        elif isinstance(exception, HttpError) and status_code in self.error_template_names:
            context = {'error': {'status_code': status_code,
                                 'message': exception.message or None}}
            return self.error_view(request, context,
                                   self.error_template_names[status_code])
        # Otherwise, try to re-raise the blighter
        elif all(sys.exc_info()):
            raise
        else:
            raise exception

    def error_406(self, request, exception, *args, **kwargs):
        accept_header_parsed = MediaType.parse_accept_header(request.META.get('HTTP_ACCEPT', ''))
        accept_header_parsed.sort(reverse=True)
        accept_header_parsed = map(unicode, accept_header_parsed)
        context = {'error': {'status_code': httplib.NOT_ACCEPTABLE,
                             'tried_mimetypes': exception.tried_mimetypes,
                             'available_renderers': [self.renderer_for_context(request, r) for r in self.conneg.renderers],
                             'format_parameter_name': self._format_override_parameter,
                             'format_parameter': request.REQUEST.get(self._format_override_parameter),
                             'format_parameter_parsed': request.REQUEST.get(self._format_override_parameter, '').split(','),
                             'accept_header': request.META.get('HTTP_ACCEPT'),
                             'accept_header_parsed': accept_header_parsed}}
        return self.error_view(request, context,
                               self.error_template_names[httplib.NOT_ACCEPTABLE])

# For backwards compatibility
ErrorCatchingView = ContentNegotiatedView

class HTMLView(ContentNegotiatedView):
    _default_format = 'html'

    @renderer(format="html", mimetypes=('text/html', 'application/xhtml+xml'), priority=1, name='HTML')
    def render_html(self, request, context, template_name):
        template_name = self.join_template_name(template_name, 'html')
        if template_name is None:
            return NotImplemented
        try:
            return render_to_response(template_name,
                                      context, context_instance=RequestContext(request),
                                      mimetype='text/html')
        except TemplateDoesNotExist:
            return NotImplemented

class TextView(ContentNegotiatedView):
    @renderer(format="txt", mimetypes=('text/plain',), priority=1, name='Plain text')
    def render_text(self, request, context, template_name):
        template_name = self.join_template_name(template_name, 'txt')
        if template_name is None:
            return NotImplemented
        try:
            return render_to_response(template_name,
                                      context, context_instance=RequestContext(request),
                                      mimetype='text/plain')
        except TemplateDoesNotExist:
            return NotImplemented

try:
    import json
except ImportError:
    try:
        import simplejson as json
    except ImportError:
        pass

# Only define if json is available.
if 'json' in locals():
    class JSONView(ContentNegotiatedView):
        _json_indent = 2

        def preprocess_context_for_json(self, context):
            return context

        def simplify_for_json(self, value):
            if inspect.ismethod(getattr(value, 'simplify_for_json', None)):
                return value.simplify_for_json(self.simplify_for_json)
            if isinstance(value, datetime.datetime):
                if value.tzinfo:
                    value = value.astimezone(utc)
                return time.mktime(value.timetuple()) * 1000
            if isinstance(value, (list, tuple)):
                items = []
                for item in value:
                    item = self.simplify_for_json(item)
                    if item is not NotImplemented:
                        items.append(item)
                return items
            if isinstance(value, dict):
                items = {}
                for key, item in value.iteritems():
                    item = self.simplify_for_json(item)
                    if item is not NotImplemented:
                        items[unicode(key)] = item
                return items
            elif type(value) in (str, unicode, int, float, long, bool):
                return value
            elif value is None:
                return value
            else:
                logger.warning("Failed to simplify object of type %r", type(value))
                return NotImplemented

        def simplify(self, value):
            warnings.warn("JSONView.simplify() has been renamed to simplify_for_json")
            return self.simplify_for_json(value)

        @renderer(format='json', mimetypes=('application/json',), name='JSON')
        def render_json(self, request, context, template_name):
            context = self.preprocess_context_for_json(context)
            return http.HttpResponse(json.dumps(self.simplify_for_json(context), indent=self._json_indent),
                                     mimetype="application/json")

    class JSONPView(JSONView):
        # The query parameter to look for the callback name
        _default_jsonp_callback_parameter = 'callback'
        # The default callback name if none is provided
        _default_jsonp_callback = 'callback'

        # Overridden to return JSONP if there's a callback parameter
        @renderer(format='json', mimetypes=('application/json',), name='JSON')
        def render_json(self, request, context, template_name):
            if self._default_jsonp_callback_parameter in request.GET:
                return self.render_js(request, context, template_name)
            else:
                return super(JSONPView, self).render_json(request, context, template_name)

        @renderer(format='js', mimetypes=('text/javascript', 'application/javascript'), name='JavaScript (JSONP)')
        def render_js(self, request, context, template_name):
            context = self.preprocess_context_for_json(context)
            callback_name = request.GET.get(self._default_jsonp_callback_parameter,
                                            self._default_jsonp_callback)

            return http.HttpResponse('%s(%s);' % (callback_name, json.dumps(self.simplify_for_json(context), indent=self._json_indent)),
                                     mimetype="application/javascript")

class ErrorView(HTMLView, JSONPView, TextView):
    _force_fallback_format = ('html', 'json')
    def get(self, request, context, template_name):
        self.context.update(context)
        self.template_name = template_name
        self.context['error']['response'] = httplib.responses[context['error']['status_code']]
        self.context['status_code'] = context['error']['status_code']
        return self.render()
    post = delete = put = get



import imp

INSTALLED_APPS = (
    'django_conneg',
)

# Use django_jenkins if it's installed.
try:
    imp.find_module('django_jenkins')
except ImportError:
    pass
else:
    INSTALLED_APPS += ('django_jenkins',)

DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3'}}

LOGIN_URL = '/login/'

ROOT_URLCONF = 'django_conneg.tests.urls'

BASIC_AUTH_ALLOW_HTTP = True


import re

from django.http import HttpResponseRedirect

class HttpResponseSeeOther(HttpResponseRedirect):
    status_code = 303

class HttpResponseTemporaryRedirect(HttpResponseRedirect):
    status_code = 307

class HttpError(Exception):
    def __init__(self, status_code=None, message=None):
        if status_code:
            self.status_code = status_code
        super(HttpError, self).__init__(message)

class HttpNotAcceptable(HttpError):
    status_code = 406
    def __init__(self, tried_mimetypes):
        self.tried_mimetypes = tried_mimetypes

class HttpBadRequest(HttpError):
    status_code = 400

class MediaType(object):
    """
    Represents a parsed internet media type.
    """

    _MEDIA_TYPE_RE = re.compile(r'(\*/\*)|(?P<type>[^/]+)/(\*|((?P<subsubtype>[^+]+)\+)?(?P<subtype>.+))')
    def __init__(self, value, priority=0):
        value = unicode(value).strip()
        media_type = value.split(';')
        media_type, params = media_type[0].strip(), dict((i.strip() for i in p.split('=', 1)) for p in media_type[1:] if '=' in p)

        mt = self._MEDIA_TYPE_RE.match(media_type)
        if not mt:
            raise ValueError("Not a correctly formatted internet media type (%r)" % media_type)
        mt = mt.groupdict()

        try:
            self.quality = float(params.pop('q', 1))
        except ValueError:
            self.quality = 1

        self.type = mt.get('type'), mt.get('subtype'), mt.get('subsubtype')
        self.specifity = len([t for t in self.type if t])
        self.params = params
        self.value = value
        self.priority = priority

    def __unicode__(self):
        return self.value

    def __gt__(self, other):
        if self.quality != other.quality:
            return self.quality > other.quality

        if self.specifity != other.specifity:
            return self.specifity > other.specifity

        for key in other.params:
            if self.params.get(key) != other.params[key]:
                return False

        return len(self.params) > len(other.params)

    def __lt__(self, other):
        return other > self

    def __eq__(self, other):
        return self.quality == other.quality and self.type == other.type and self.params == other.params
    def __hash__(self):
        return hash(hash(self.quality) + hash(self.type) + hash(tuple(sorted(self.params.iteritems()))))
    def __ne__(self, other):
        return not self.__eq__(other)
    def equivalent(self, other):
        """
        Returns whether two MediaTypes have the same overall specifity.
        """
        return not (self > other or self < other)

    def __cmp__(self, other):
        if self > other:
            return 1
        elif self < other:
            return -1
        else:
            return 0

    def __repr__(self):
        return "%s(%r)" % (type(self).__name__, self.value)

    def provides(self, imt):
        """
        Returns True iff the self is at least as specific as other.

        Examples:
        application/xhtml+xml provides application/xml, application/*, */*
        text/html provides text/*, but not application/xhtml+xml or application/html
        """
        return self.type[:imt.specifity] == imt.type[:imt.specifity]

    @classmethod
    def resolve(cls, accept, available_renderers):
        """
        Resolves a list of accepted MediaTypes and available renderers to the preferred renderer.

        Call as MediaType.resolve([MediaType], [renderer]).
        """
        assert isinstance(available_renderers, tuple)
        accept = sorted(accept)

        renderers, seen = [], set()

        accept_groups = [[accept.pop()]]
        for imt in accept:
            if imt.equivalent(accept_groups[-1][0]):
                accept_groups[-1].append(imt)
            else:
                accept_groups.append([imt])

        for accept_group in accept_groups:
            for renderer in available_renderers:
                if renderer in seen:
                    continue
                for mimetype in renderer.mimetypes:
                    for imt in accept_group:
                        if mimetype.provides(imt):
                            renderers.append(renderer)
                            seen.add(renderer)
                            break

        return renderers

    @classmethod
    def parse_accept_header(cls, accept):
        media_types = []
        for media_type in accept.split(','):
            try:
                media_types.append(MediaType(media_type))
            except ValueError:
                pass
        return media_types


import base64
import httplib
import urlparse

from django.conf import settings
from django.contrib.auth import authenticate
from django_conneg.views import HTMLView, JSONPView, TextView

class UnauthorizedView(HTMLView, JSONPView, TextView):
    _force_fallback_format = 'txt'
    template_name = 'conneg/unauthorized'

    def get(self, request):
        self.context.update({'status_code': httplib.UNAUTHORIZED,
                             'error': 'You need to be authenticated to perform this request.'})
        return self.render()
    post = put = delete = get

class InactiveUserView(HTMLView, JSONPView, TextView):
    _force_fallback_format = 'txt'
    template_name = 'conneg/inactive_user'

    def get(self, request):
        self.context.update({'status_code': httplib.FORBIDDEN,
                             'error': 'Your account is inactive.'})
        return self.render()
    post = put = delete = get

class BasicAuthMiddleware(object):
    """
    Sets request.user if there are valid basic auth credentials on the
    request, and turns @login_required redirects into 401 responses for
    non-HTML responses.
    """

    allow_http = getattr(settings, 'BASIC_AUTH_ALLOW_HTTP', False) or settings.DEBUG

    unauthorized_view = staticmethod(UnauthorizedView.as_view())
    inactive_user_view = staticmethod(InactiveUserView.as_view())

    def process_request(self, request):
        # Ignore if user already authenticated
        if request.user.is_authenticated():
            return

        # Don't do anything for unsecure requests, unless DEBUG is on
        if not self.allow_http and not request.is_secure():
            return

        # Parse the username and password out of the Authorization
        # HTTP header and set request.user if we find an active user.
        # We don't use auth.login, as the authorization is only valid
        # for this one request.
        authorization = request.META.get('HTTP_AUTHORIZATION')
        if not authorization or not authorization.startswith('Basic '):
            return
        try:
            credentials = base64.b64decode(authorization[6:]).split(':', 1)
        except TypeError:
            return
        if len(credentials) != 2:
            return
        user = authenticate(username=credentials[0], password=credentials[1])
        if user and user.is_active:
            request.user = user
        elif user and not user.is_active:
            return self.inactive_user_view(request)
        else:
            return self.unauthorized_view(request)

    def process_response(self, request, response):
        process = False

        # Don't do anything for unsecure requests, unless DEBUG is on
        if not self.allow_http and not request.is_secure():
            return response

        if response.status_code == httplib.UNAUTHORIZED:
            process = True
        elif response.status_code == httplib.FOUND:
            # Don't return a 401 if the view wasn't handled by a
            # ContentNegotiatedView.
            renderers = getattr(request, 'renderers', None)
            if renderers and renderers[0].format != 'html':
                location = urlparse.urlparse(response['Location'])
                if location.path == settings.LOGIN_URL:
                    response = self.unauthorized_view(request)
                    process = True

        if not process:
            return response

        realm = getattr(settings, 'BASIC_AUTH_REALM', request.META.get('HTTP_HOST', 'restricted'))

        authenticate = response.get('WWW-Authenticate', None)
        if authenticate:
            authenticate = 'Basic realm="%s", %s' % (realm, authenticate)
        else:
            authenticate = 'Basic realm="%s"' % realm
        response['WWW-Authenticate'] = authenticate

        return response



from collections import defaultdict
import inspect
import weakref

from django_conneg.http import MediaType

class Renderer(object):
    def __init__(self, func, format, mimetypes=(), priority=0, name=None, test=None, instance=None, owner=None):
        self.func = func
        self.test = test or (lambda s,r,c,t: True)
        if instance:
            self.func = func.__get__(instance, owner)
            self.test = test.__get__(instance, owner)

        self.is_renderer = True
        self.format = format
        self.mimetypes = set(MediaType(mimetype, priority) for mimetype in mimetypes)
        self.name = name
        self.priority = priority

        self.is_bound = instance is not None

    def __get__(self, instance, owner=None):
        return Renderer(self.func, self.format, self.mimetypes, self.priority, self.name, self.test, instance, owner)
    def __call__(self, *args, **kwargs):
        return self.func(*args, **kwargs)

    def __repr__(self):
        if self.is_bound:
            return "<bound renderer {0}.{1} of {2}>".format(self.func.im_class.__name__ or '?',
                                                            self.func.__name__,
                                                            self.func.__self__)
        else:
            return "<unbound renderer {0}>".format(self.func.__name__)

class Conneg(object):
    _memo_by_class = weakref.WeakKeyDictionary()

    def __init__(self, renderers=None, obj=None):
        self.renderers_by_format = defaultdict(list)
        self.renderers_by_mimetype = defaultdict(list)

        if renderers is not None:
            renderers = list(renderers)
        elif obj:
            cls = type(obj) if not isinstance(obj, type) else obj
            renderers = self._memo_by_class.get(cls)
            if renderers is None:
                # This is about as much memoization as we can do. We keep
                # the renderers unbound for now
                renderers = []
                for name in dir(cls):
                    try:
                        value = getattr(obj, name)
                    except AttributeError:
                        continue
                    if isinstance(value, Renderer):
                        renderers.append(value)
                self._memo_by_class[cls] = renderers

            # Bind the renderers to this instance. See
            # http://stackoverflow.com/a/1015405/613023 for an explanation.
            renderers = [r.__get__(obj, cls) for r in renderers]

        for renderer in renderers:
            if renderer.mimetypes is not None:
                for mimetype in renderer.mimetypes:
                    self.renderers_by_mimetype[mimetype].append(renderer)
                self.renderers_by_format[renderer.format].append(renderer)

        # Order all the renderers by priority
        renderers.sort(key=lambda renderer:-renderer.priority)
        self.renderers = tuple(renderers)

    def get_renderers(self, request, context=None, template_name=None,
                      accept_header=None, formats=None, default_format=None, fallback_formats=None):
        """
        Returns a list of renderer functions in the order they should be tried.
        
        Tries the format override parameter first, then the Accept header. If
        neither is present, attempt to fall back to self._default_format. If
        a fallback format has been specified, we try that last.
        """
        if formats:
            renderers, seen_formats = [], set()
            for format in formats:
                if format in self.renderers_by_format and format not in seen_formats:
                    renderers.extend(self.renderers_by_format[format])
                    seen_formats.add(format)
        elif accept_header:
            accepts = MediaType.parse_accept_header(accept_header)
            renderers = MediaType.resolve(accepts, self.renderers)
        elif default_format:
            renderers = self.renderers_by_format[default_format]
        else:
            renderers = []

        fallback_formats = fallback_formats if isinstance(fallback_formats, (list, tuple)) else (fallback_formats,)
        for format in fallback_formats:
            for renderer in self.renderers_by_format[format]:
                if renderer not in renderers:
                    renderers.append(renderer)

        if context is not None and template_name:
            renderers = [r for r in renderers if r.test(request, context, template_name)]

        return renderers

    def __add__(self, other):
        if not isinstance(other, Conneg):
            other = Conneg(obj=other)
        return Conneg(self.renderers + other.renderers)


try:
    from pytz import utc
except ImportError:
    import datetime

    class _UTC(datetime.tzinfo):
        def utcoffset(self, dt):
            return datetime.timedelta(0)
        def dst(self, dt):
            return datetime.timedelta(0)
        def tzname(self, dt):
            return "UTC"
    
    utc = _UTC()


from django.conf.urls.defaults import patterns, url
from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django_conneg.views import HTMLView, JSONView

class OptionalAuthView(HTMLView, JSONView):
    _force_fallback_format = ('html', 'json')
    def get(self, request):
        response = self.render()
        response.is_authenticated = request.user.is_authenticated()
        return response

class LoginRequiredView(HTMLView, JSONView):
    _force_fallback_format = ('html', 'json')
    @method_decorator(login_required)
    def get(self, request):
        response = self.render()
        response.is_authenticated = request.user.is_authenticated()
        return response

urlpatterns = patterns('',
    url(r'^optional-auth/$', OptionalAuthView.as_view()),
    url(r'^login-required/$', LoginRequiredView.as_view()),
)


from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES
import os

from django_conneg import __version__

#################################
# BEGIN borrowed from Django    #
# licensed under the BSD        #
# http://www.djangoproject.com/ #
#################################

def fullsplit(path, result=None):
    """
Split a pathname into components (the opposite of os.path.join) in a
platform-neutral way.
"""
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk('django_conneg'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

#################################
# END borrowed from Django      #
#################################


# Idea borrowed from http://cburgmer.posterous.com/pip-requirementstxt-and-setuppy
install_requires, dependency_links = [], []
for line in open('requirements.txt'):
    line = line.strip()
    if line.startswith('-e'):
        dependency_links.append(line[2:].strip())
    elif line:
        install_requires.append(line)


setup(
    name='django-conneg',
    description="An implementation of content-negotiating class-based views for Django",
    author='Oxford University Computing Services',
    author_email='infodev@oucs.ox.ac.uk',
    version=__version__,
    packages=packages,
    license='BSD',
    url='https://github.com/oucs/django-conneg',
    long_description=open('README.rst').read(),
    classifiers=['Development Status :: 4 - Beta',
                 'Framework :: Django',
                 'License :: OSI Approved :: BSD License',
                 'Natural Language :: English',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content'],
    keywords=['REST', 'University of Oxford', 'content negotiation', 'Accept header', 'Django'],
    data_files=data_files,
    install_requires=install_requires,
    dependency_links=dependency_links,
)


